package cn.com.qmth.am.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.core.ai.model.llm.AutoScoreRequest;
import com.qmth.boot.core.ai.model.llm.AutoScoreResult;
import com.qmth.boot.core.ai.model.llm.ChatRole;
import com.qmth.boot.core.retrofit.exception.RetrofitResponseError;

import cn.com.qmth.am.bean.ds.ChatResult;
import cn.com.qmth.am.bean.ds.ReqMessages;
import cn.com.qmth.am.config.SysProperty;
import cn.com.qmth.am.service.DsMarkingService;
import cn.com.qmth.am.utils.FreeMarkerUtil;
import cn.com.qmth.am.utils.HttpMethod;
import cn.com.qmth.am.utils.OKHttpUtil;
import okhttp3.Response;

@Service
public class DsMarkingServiceImpl implements DsMarkingService {

    @Autowired
    private SysProperty sysProperty;

    private static final Logger log = LoggerFactory.getLogger(DsMarkingService.class);

    @SuppressWarnings("deprecation")
    public static void main(String[] args) {

        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer 7dac2f2166994b8f9c6de0a8eff2814c");
        Response resp = null;
        try {
            resp = OKHttpUtil.call(HttpMethod.POST, "http://39.174.90.3:31091/spiritx-api/v1/chat/completions", headers,
                    "{\"model\":\"deepseek-r1-distill-qwen-32b-awq\",\"messages\":[{\"role\":\"user\",\"content\":\"你是谁？\"}]}");
            if (resp.code() != 200) {
                throw new RuntimeException("err :" + resp.body().string());
            } else {
                System.out.println("成功处理：" + resp.body().string());
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            IOUtils.closeQuietly(resp);
        }
    }

    @Override
    public AutoScoreResult autoScore(AutoScoreRequest request) {
        String question = FreeMarkerUtil.getDsMarkingReq(request);
        ReqMessages dreq = new ReqMessages(sysProperty.getMarkingModel());
        dreq.addMsg(ChatRole.user, question);
        String res = chat(dreq);
        ChatResult result = JSONObject.parseObject(res, ChatResult.class);
        String text = result.getChoices().stream().filter(choice -> choice.getMessage().getRole() == ChatRole.assistant)
                .map(choice -> choice.getMessage().getContent()).findFirst().orElse("");
        try {
            AutoScoreResult scoreResult = new AutoScoreResult();
            // 依据总分与步骤分计算最大精度
            int scale = Math.max(getDecimalPlaces(request.getIntervalScore()),
                    getDecimalPlaces(request.getTotalScore()));
            int stepCount = request.getStandardAnswer().size();
            String scoreStr = text.substring(text.lastIndexOf("\n") + 1).trim();
            String[] scores = StringUtils.split(scoreStr.replaceAll("，", ",").replaceAll("[0-9]\\.", ""), ",");
            double[] scoreArray = new double[stepCount];
            for (int i = 0; i < stepCount; i++) {
                // 根据得分率与步骤总分计算实际得分，按最大精度保留小数位数
                double score = BigDecimal
                        .valueOf(Math.min(Integer.parseInt(scores[i].trim()), 100)
                                * request.getStandardAnswer().get(i).getScore())
                        .divide(BigDecimal.valueOf(100), scale, RoundingMode.HALF_UP).doubleValue();
                scoreArray[i] = score;
            }
            scoreResult.setStepScore(scoreArray);
            scoreResult.setTotalScore(Arrays.stream(scoreArray).mapToObj(BigDecimal::new)
                    .reduce(BigDecimal.ZERO, BigDecimal::add).doubleValue());
            return scoreResult;
        } catch (Exception e) {
            log.error(e.getMessage() + " | " + res);
            return null;
        }
    }

    private int getDecimalPlaces(double value) {
        return Math.max(0, BigDecimal.valueOf(value).stripTrailingZeros().scale());
    }

    @SuppressWarnings("deprecation")
    private String chat(ReqMessages dreq) {

        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer " + sysProperty.getMarkingKey());
        Response resp = null;
        try {
            resp = OKHttpUtil.call(HttpMethod.POST, sysProperty.getMarkingServer(), headers,
                    JSONObject.toJSONString(dreq));
            if (resp.code() != 200) {
                throw new RetrofitResponseError(resp.code(), resp.body().string());
            } else {
                return resp.body().string();
            }
        } catch (Exception e) {
            throw new RetrofitResponseError(resp.code(), e.getMessage(), e);
        } finally {
            IOUtils.closeQuietly(resp);
        }
    }
}
